"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const errors_1 = require("./errors");
const yml_1 = require("./yml");
const commander_1 = require("commander");
function getCustomConfigPath(cliFlags) {
    const configFile = cliFlags.config;
    if (configFile) {
        const configPath = path_1.resolve(process.cwd(), configFile);
        if (!fs_1.existsSync(configPath)) {
            throw new errors_1.DetailedError(`Config ${configPath} does not exist`, `
        Config ${configPath} does not exist.

          $ gql-gen --config ${configPath}

        Please make sure the --config points to a correct file.
      `);
        }
        return configPath;
    }
    return null;
}
function loadAndParseConfig(filepath) {
    const ext = filepath.substr(filepath.lastIndexOf('.') + 1);
    switch (ext) {
        case 'yml':
            return yml_1.parseConfigFile(fs_1.readFileSync(filepath, 'utf-8'));
        case 'json':
            return JSON.parse(fs_1.readFileSync(filepath, 'utf-8'));
        case 'js':
            return require(path_1.resolve(process.cwd(), filepath));
        default:
            throw new errors_1.DetailedError(`Extension '${ext}' is not supported`, `
        Config ${filepath} couldn't be parsed. Extension '${ext}' is not supported.
      `);
    }
}
function createConfig(argv = process.argv) {
    const cliFlags = new commander_1.Command()
        .usage('gql-gen [options]')
        .allowUnknownOption(true)
        .option('-c, --config <path>', 'Path to GraphQL codegen YAML config file, defaults to "codegen.yml" on the current directory')
        .option('-w, --watch', 'Watch for changes and execute generation automatically')
        .option('-o, --overwrite', 'Overwrites existing files')
        .parse(argv);
    const customConfigPath = getCustomConfigPath(cliFlags);
    const locations = [path_1.join(process.cwd(), './codegen.yml'), path_1.join(process.cwd(), './codegen.json')];
    if (customConfigPath) {
        locations.unshift(customConfigPath);
    }
    const filepath = locations.find(fs_1.existsSync);
    if (!filepath) {
        throw new errors_1.DetailedError(`Unable to find Codegen config file!`, `
        Please make sure that you have a configuration file under the current directory! 
      `);
    }
    const parsedConfigFile = loadAndParseConfig(filepath);
    if (cliFlags.watch === true) {
        parsedConfigFile.watch = cliFlags.watch;
    }
    if (cliFlags.overwrite === true) {
        parsedConfigFile.overwrite = cliFlags.overwrite;
    }
    return parsedConfigFile;
}
exports.createConfig = createConfig;
//# sourceMappingURL=config.js.map